/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: openh323-telephone
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - localized
  *       - gives proper hints about being opaque
  * 0.0.2 (2002-03-02, Marcus Wolschon<Suran@gmx.net>)
  *       - uses AR-panel
  * -------------------------------------------------------------------
  * TODO:
  *       - more testing
  */
package jaugment2.openh323telephone;

  //JDK
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.MarshalledObject;
import java.lang.StringBuffer;
import java.lang.Package;
import java.lang.Double;
import java.io.Serializable;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.lang.IllegalStateException;
import java.util.HashSet;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;
import java.util.ListIterator;
import java.util.LinkedList;
import java.beans.*;
   //Swing
import javax.swing.*;
import javax.swing.JScrollPane;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import java.awt.event.*;
import java.awt.*;
  //JNDI
import javax.naming.NamingEnumeration;
import javax.naming.directory.SearchResult;
import javax.naming.directory.Attributes;
import javax.naming.directory.Attribute;
  //JINI
import net.jini.core.entry.Entry;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
import net.jini.core.event.EventRegistration;
import net.jini.lease.LeaseRenewalManager;
import net.jini.core.lease.Lease;
import net.jini.core.lookup.ServiceItem;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JComponentFactory;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
import net.jini.lookup.ui.attribute.RequiredPackages;
  //project
import jaugment2.UI.swingUI.SwingUIRepresentable;
import jaugment2.helpers.Logger;
import jaugment2.directory.Directory;
import jaugment2.directory.Directory.*;
import jaugment2.openh323telephone.TelephoneService.TelephoneServiceEvent;
   // AR-UI
import jaugment2.UI.ARUI.jini.ARUIWidgetFactory;
import jaugment2.UI.ARUI.widgets.*;

/** A telephone using openh323
  */
public class TelephoneARRepresentation  extends jaugment2.UI.ARUI.widgets.Panel implements Entry, RemoteEventListener, ARUIWidgetIF
{

 private static jaugment2.helpers.Translator translate = new jaugment2.helpers.Translator(TelephoneARRepresentation.class);

 private static final boolean debugMessages = true;
 protected TelephoneProxyIF proxy;
 private LinkedList directories = new LinkedList();


  TelephoneButton jButton_call_hangup = new TelephoneButton();
  JComboBox jComboBox_to;
  ARButton jButton3;

  /** React to events from our service.
    */
  public void notify(RemoteEvent evt) throws UnknownEventException, RemoteException
  {
   Logger.debug.println("DEBUG[TelephoneARRepresentation] notify()");
   if(!(evt instanceof TelephoneServiceEvent))
     throw(new UnknownEventException("expected TelephoneServiceEvent"));

   switch((int)((TelephoneServiceEvent)evt).getID())
    {
     case TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_FOUND:
       {
        directories.add(((TelephoneServiceEvent)evt).obj);
        searchDirectories(true);
        break;
       }
     case TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_GONE:
       {
        directories.remove(((TelephoneServiceEvent)evt).obj);
        break;
       }
     default:
        throw(new UnknownEventException("unknown event-ID"));
    }
  }


 /**
  * Get a UIDescriptor as needed by the serviceui-draft-standart
  */
 public static UIDescriptor getUIDescriptor()
  {
   try
     {
      return(new UIDescriptor(MainUI.ROLE,
                              JComponentFactory.TOOLKIT,
                              getServiceUIProperties(),
                              new MarshalledObject(new Factory()))
                             );
     }
   catch(IOException e)
     {
      e.printStackTrace(Logger.err);
     }
   return(null);
  }



  /**
   * Gets a serviceui-draft-standart-compactible attribute-set of this representation
   */
  static Set getServiceUIProperties()
  {
   HashSet set = new HashSet();
   set.add("javax.swing.JComponent");
   set.add("javax.swing.JPanel");
   set.add("java.awt.Component");
   set.add("jaugment2.UI.ARUI");

   HashSet retval = new HashSet();
   retval.add(new UIFactoryTypes(set));

   HashMap map = new HashMap();
   map.put("javax.swing", Package.getPackage("javax.swing").getSpecificationVersion());
   map.put("jaugment2.UI.ARUI", (Package.getPackage("jaugment2.UI.ARUI").getSpecificationVersion()==null?"0":Package.getPackage("jaugment2.UI.ARUI").getSpecificationVersion()));
   RequiredPackages req = new RequiredPackages(map);
   retval.add(req);

   return(retval);
  }


  /**
   * A serializable factory for creating this UI
   */
  public static class Factory implements ARUIWidgetFactory, Serializable
  {


   /**
    * Returns a <CODE>JComponent</CODE>.
    */
   public ARUIWidgetIF createARUIWidget(Object roleObject)
   {
    ServiceItem item = (ServiceItem)roleObject;

    if(debugMessages)Logger.debug.println("DEBUG: [TelephoneARRepresentation.factory] getJComponent called");
    try
      {
       TelephoneARRepresentation rep = new TelephoneARRepresentation( ((TelephoneProxyIF)item.service) );
       return(rep);
      }
    catch(Exception ex)
       {
        if(debugMessages)ex.printStackTrace(Logger.debug);
        return(null);
       }

   }
  }





  private LeaseRenewalManager lrm = new LeaseRenewalManager();

  /**
   * Constructor
   */
 public TelephoneARRepresentation(TelephoneProxyIF proxy)
 {
  try
    {
     if(debugMessages)Logger.debug.println("DEBUG: [TelephoneARRepresentation] TelephoneSwingRepresentation(...)");

     this.proxy = proxy;
     Lease l = proxy.addListener(new TelephoneARRepresentationListener(this), null, TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_FOUND | TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_GONE);
     if(l != null)
       {
        lrm.renewFor(l, Lease.FOREVER, null);
       };

     this.directories = proxy.getDirectories();
     searchDirectories(true);

     jbInit();
    }
  catch(Exception e)
    {
     e.printStackTrace(Logger.err);
    }
 }


  /** Create UI.
    */
  private void jbInit() throws Exception
  {
   setBackground(new java.awt.Color( GraphicHelper.getFillColor().getRed(), GraphicHelper.getFillColor().getGreen(), GraphicHelper.getFillColor().getBlue()) );
   setOpaque(true);
   if(jComboBox_to==null)
     {
      jComboBox_to = new JComboBox(new DefaultComboBoxModel());
     }
    jButton3 = new ARButton();
    jButton3.setBackground(GraphicHelper.getFillColor());
    jButton_call_hangup.setMaximumSize(new Dimension(59, 230));
    jButton_call_hangup.setMinimumSize(new Dimension(59, 230));
    jButton_call_hangup.setPreferredSize(new Dimension(59, 230));
    jButton_call_hangup.setText(translate.getString("call"));
    jButton_call_hangup.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButton_call_hangup_actionPerformed(e);
      }
    });
    jButton3.setText(translate.getString("search"));
    jButton3.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton3_actionPerformed(e);
      }
    });
    jComboBox_to.setEditable(true);
    this.add(jButton_call_hangup, null);
    this.add(jComboBox_to, null);
    this.add(jButton3, null);
  }


  private boolean calling = false;

  void jButton_call_hangup_actionPerformed(ActionEvent evt)
  {
   if(calling)
   try
     {
      Object to = jComboBox_to.getSelectedItem();
      proxy.makeCall(to.toString());
      calling = true;
      jButton_call_hangup.setText(translate.getString("hangup"));
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
   else
   try
     {
      proxy.hangup();
      calling = false;
      jButton_call_hangup.setText(translate.getString("call"));
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }



  void jButton3_actionPerformed(ActionEvent e)
  {
   searchDirectories(false);
  }
  //----------------------------------------------------------







  private void searchDirectories(boolean local)
  {
   if(jComboBox_to==null)
     {
      jComboBox_to = new JComboBox(new DefaultComboBoxModel());
     }
   TelephoneSwingRepresentation.searchDirectories(local, (DefaultComboBoxModel)jComboBox_to.getModel(), directories);
  }


//---------------------------------------------------


 public IconSlot getFreeIconSlot()
 {
  return null;
 }
 public Block getFreeBlock()
 {
  return null;
 }

//----------------------------------------------------


 private static final int framewidth = 2;
 private static final int edgesize = 5;
 public  static final java.awt.Color bordercolor =  GraphicHelper.getInnerBorderColor(); //new java.awt.Color(200, 20, 20);


private static class TelephoneButton extends JButton
{
 public TelephoneButton()
 {
  setOpaque(false);
 }
 public void paint(Graphics g)
 {
   try
     {
      int width = getWidth()-framewidth/2;
      int height = getHeight()-framewidth/2;
//      g.setColor(GraphicHelper.getFillColor());
//      g.fillRect(0,0,getWidth(), getHeight());
      java.awt.Point p[] = new java.awt.Point[]
         {
          new java.awt.Point(edgesize+framewidth/2, framewidth/2),

          new java.awt.Point(width-edgesize, framewidth/2),
          new java.awt.Point(width, edgesize+framewidth/2),

          new java.awt.Point(width, (3*height/12)-edgesize + (framewidth/2)),
          new java.awt.Point(width-edgesize, (3*height/12) + (framewidth/2)),

          new java.awt.Point(edgesize+9*width/12, (3*height/12) + (framewidth/2)),
          new java.awt.Point(9*width/12, (3*height/12) + edgesize + (framewidth/2)),

          new java.awt.Point((9*width/12), (9*height/12) -edgesize + (framewidth/2)),
          new java.awt.Point((9*width/12) + edgesize, (9*height/12) + (framewidth/2)),

          new java.awt.Point(width-edgesize, (9*height/12) + (framewidth/2)),
          new java.awt.Point(width, (9*height/12) + edgesize + (framewidth/2)),

          new java.awt.Point(width, height - edgesize),
          new java.awt.Point(width - edgesize, height),

          new java.awt.Point((framewidth/2) + edgesize, height),
          new java.awt.Point(framewidth/2, height - edgesize),

          new java.awt.Point(framewidth/2, (9*height/12) + edgesize),
          new java.awt.Point((framewidth/2) + edgesize, 9*height/12),

          new java.awt.Point((3*width/12) - edgesize, 9*height/12),
          new java.awt.Point(3*width/12, (9*height/12) - edgesize),

          new java.awt.Point(3*width/12, (3*height/12) + edgesize),
          new java.awt.Point((3*width/12) - edgesize, 3*height/12),

          new java.awt.Point((framewidth/2) + edgesize, (3*height/12)),
          new java.awt.Point((framewidth/2), (3*height/12)  - edgesize),

          new java.awt.Point(framewidth/2, (framewidth/2) + edgesize),
          new java.awt.Point((framewidth/2) + edgesize, framewidth/2),

          new java.awt.Point((framewidth/2) + edgesize + 5, framewidth/2)
         };
      GraphicHelper.drawSmoothPolyline(p, framewidth, bordercolor, g);
      int textwidth  = g.getFontMetrics().charsWidth(getText().toCharArray(), 0, getText().length());
      g.drawString(getText(), (width-textwidth)/2, height/2);
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
 }
}


}
