/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: openh323-telephone
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-17, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2000-12-17, Marcus Wolschon<Suran@gmx.net>)
  *       - getDirectories
  *       - addListener
  * 0.0.3 (2000-12-19, Marcus Wolschon<Suran@gmx.net>)
  *       - TelephoneProxyIF
  * -------------------------------------------------------------------
  * TODO:
  *       - allow incomming calls
  *       - allow video via ppm-files
  */
package jaugment2.openh323telephone;

  //JDK
import java.rmi.server.UnicastRemoteObject;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.io.Serializable;
import java.lang.IllegalStateException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.util.LinkedList;
//import java.net.URL;
  //JINI
import net.jini.core.event.RemoteEventListener;
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.Lease;
import net.jini.admin.Administrable;
  //project
import jaugment2.jini.simpleService.simpleAdministrator;
import jaugment2.helpers.Logger;
import jaugment2.directory.Directory;

//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


/**
 * A telephone using openh323
 */
public class TelephoneProxy extends UnicastRemoteObject implements Administrable, Serializable, Remote, TelephoneProxyIF
{
 private TelephoneService service;


  TelephoneProxy(TelephoneService service ) throws RemoteException
  {
   try
     {
      this.service = service;
     }
   catch(Exception e)
     {
      Logger.err.println("Exception in TelephoneProxy::TelephoneProxy() has the Message: "+e.getMessage());
      e.printStackTrace(Logger.err);
     }
  }


  /*public void pause() throws RemoteException, IllegalArgumentException, IllegalStateException
  {
   service.pause();
  } */




 /**
   * Returns an object that implements our administration-interfaces.
   *
   * @throws java.rmi.RemoteException
   * @see JoinAdmin
   */
  public Object getAdmin() throws RemoteException
  {
   return(new simpleAdministrator(service));
  }


  //-------------- Client-functions ------------------------------

  /** Subscribe for events
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   return(service.addListener(listener, handback, registerfor));
  }

  /**
   * Make a call to "to"
   */
  public boolean makeCall(String to) throws RemoteException
  {
   return(service.makeCall(to));
  }

  public void hangup() throws RemoteException
  {
   service.hangup();
  }

  /**
   * Return the directory-services found so far
   * @returns a LinkedList of Directory
   */
  public LinkedList getDirectories() throws RemoteException
  {
   return(service.getDirectories());
  }

}