/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: openh323-telephone
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-17, Marcus Wolschon<Suran@gmx.net>
  *       - created
  *       - calling works
  * 0.0.2 (2000-12-17, Marcus Wolschon<Suran@gmx.net>
  *       - uses ExclusiveRessourceUserEntry
  *       - claims the ressource uppon makeCall
  *       - addListener
  *       - uses directory-services
  * 0.0.3 (2002-02-24, Marcus Wolschon<Suran@gmx.net>
  *      - stop, restart,... super.* to properly fire events
  * -------------------------------------------------------------------
  * TODO:
  *       - allow incomming calls
  *       - allow video via ppm-files
  */

package jaugment2.openh323telephone;

  //JDK
import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.util.Properties;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Set;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Enumeration;
import java.lang.Process;
import java.lang.IllegalArgumentException;
import java.lang.IllegalStateException;
import java.lang.StringBuffer;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.StringTokenizer;
  //JINI
import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.lease.landlord.Landlord.RenewResults;
import com.sun.jini.lease.landlord.LocalLandlord;
import com.sun.jini.lease.landlord.LandlordLeaseFactory;
import com.sun.jini.lease.landlord.LandlordUtil;
import com.sun.jini.lease.landlord.LandlordLease;
import com.sun.jini.lease.landlord.LeasedResource;
import com.sun.jini.lease.landlord.LandlordLease.Factory;
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
//import net.jini.core.event.EventRegistration;
import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JComponentFactory;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
  //project
import jaugment2.jini.simpleService.*;
import jaugment2.jini.ExclusiveRessourceUserEntry;
import jaugment2.jini.LookupEventListener;
import jaugment2.jini.LookupEventEvent;
import jaugment2.directory.Directory;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;
import jaugment2.helpers.Logger;


//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper

/**
 * A telephone using openh323
 */
public class TelephoneService extends simpleService implements Remote, LookupEventListener
{

 private static final boolean debugMessages = true;

  private Hashtable directories = new Hashtable();

  private class ExclusiveUsersEntry
  {
   public ExclusiveRessourceUserEntry erue;
   public Object proxy;
   public ExclusiveUsersEntry(ExclusiveRessourceUserEntry erue, Object proxy)
   {
    this.erue = erue;
    this.proxy = proxy;
   }
   public void claimRessource()
   {
    erue.freeRessource(proxy); //we don't care about the result
   }
  };

  /**
   * Maps ServiceID to ExclusiveUsersEntry for
   * services using the same exclusing ressource we do
   */
  private Hashtable exclusiveUsers = new Hashtable();

  public void serviceFoundEvent(LookupEventEvent e)
  {
   if(e==null)
     return;
   if(e.item==null)
     return;

   // look for ExclusiveRessourceUserEntrys with the same ressource we use
   Entry entries[] = e.item.attributeSets;
   if(entries != null)
     for(int i=0; i<entries.length; i++)
        if(entries[i] instanceof ExclusiveRessourceUserEntry)    //put them into exclusiveUsers
        if(((ExclusiveRessourceUserEntry)entries[i]).getRessourcename().equals(persistentState.getProperty("exclusiveRessourceName")))
          exclusiveUsers.put(e.item.serviceID, new ExclusiveUsersEntry((ExclusiveRessourceUserEntry)entries[i], e.item.service));
        {
       }

   if(e.item.service instanceof Directory)
     {
      if(!this.directories.contains(e.item.serviceID))
        {
         if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService] found a new Directory");
         this.directories.put(e.item.serviceID, (Directory)e.item.service);
           // inform our listeners, that a Directory has been found
         fireEvent(jaugment2.openh323telephone.TelephoneService.TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_FOUND, e.item.service);
        }
     }

  };

  public void serviceGoneEvent(LookupEventEvent e)
  {
   exclusiveUsers.remove(e.item.serviceID);
   if(this.directories.contains(e.item.serviceID))
     {
      if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService] lost a Directory");
      this.directories.remove(e.item.serviceID);
      // inform our listeners, that a Directory has been found
      fireEvent(jaugment2.openh323telephone.TelephoneService.TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_GONE, e.item.service);
      }

  };

  public void serviceChangedEvent(LookupEventEvent e)
  {

  };


 //---------------------------------------------------------------------


/**
  * Class for events thrown here
  */
 public static class TelephoneServiceEvent extends RemoteEvent
 {
  public static final int EVID_DIRECTORY_SERVICE_FOUND = 1;
  public static final int EVID_DIRECTORY_SERVICE_GONE = 2;
  public Object obj;

  public TelephoneServiceEvent(Object proxy, long eventID, long seqNum, MarshalledObject handback, Object obj)
  {
   super(proxy, eventID, seqNum, handback);
   this.obj = obj;
  }

 }

  private LinkedList listenerRegistrations[] = new LinkedList[]{null,
                                                                new LinkedList(),
                                                                new LinkedList()
                                                               };

  /**
   * Helper-class for our Landlord.
   */
  private LeaseListener simpleLeaselistener = new LeaseListener()
  {
   public void leaseCanceled(int type, Object reg, int newcount)
   {
//    EventRegistration reg2 = (EventRegistration)reg;
    //TODO
   };
  };

  private SimpleLandlord landLord;


  private void fireEvent(int type, Object obj)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService:fireEvent] I have "+listenerRegistrations[type].size()+" registrations");
   for(int i=0; i<listenerRegistrations[type].size(); i++)
     {
      try
        {                     //TODO: do this asyncronously
         EventRegistration entry = (EventRegistration)listenerRegistrations[type].get(i);
         if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService:fireEvent] delivering event type="+type+" subscribedFor="+entry.subscribedFor+" entry.subscribedFor & type="+(entry.subscribedFor & type));
         if( (entry.subscribedFor & type) != 0)
         entry.listener.notify(
                               new TelephoneServiceEvent(getProxy(),
                                                     type,
                                                     entry.seq++,
                                                     entry.handback,
                                                     obj
                                                    )
                              );
        }
      catch(RemoteException ex)
        {
         Logger.err.println("ERROR: [TelephoneService:fireEvent] could not deliver event");
         ex.printStackTrace(Logger.err);
        }
      catch(UnknownEventException ex)
        {
         Logger.err.println("ERROR: [TelephoneService:fireEvent] could not deliver event - the receiver does not know it");
         ex.printStackTrace(Logger.err);
        }
     }
  }

  /** Subscribe for events
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   if(listener==null)
     return(null);

   try
     {
      EventRegistration reg = new EventRegistration(listener, handback);
      Lease   l = landLord.grantLease(0, reg, 2*60*10);

      if(l != null)
        {
          // tokenize the expression
         if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService] addListener expr=\""+registerfor+"\"");

         for(int i=1; i<(2*registerfor) && (i<listenerRegistrations.length); i=i*2)
            if((i & registerfor) == i)
             {
              if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService] addListener expr=\""+registerfor+"\" i="+i);
              listenerRegistrations[i].add(reg);
             }
        }

      return(l);
     }
   catch(Exception ex)
     {
      ex.printStackTrace(Logger.err);
     }
   return(null);
  }





  //-------------------- simple Service

 /** restart this service once it has been stopped
   * @see stop
   */
 public void restart() throws Exception
 {
  super.restart();
 }

 /** stop this service. If exit is false all references to this
   * service are still valid but it has deregistered everywhere.
   * (this includes the RMI-activation-daemon)
   * @see restart
   * @see destroy
   * @param exit stop running
   */
 public void stop(boolean exit) throws Exception
 {
  super.stop(exit);
 }

 /** stop this service, destroy any persistent storage and exit
   */
 public void destroy() throws RemoteException
 {
  super.destroy();
 }


  //-------------------- Constructor
  public TelephoneService() throws RemoteException
  {
   this(null);
  };


  TelephoneProxy proxy;

  public TelephoneService( LookupLocator firstLocator) throws RemoteException
  {
//   landLord = new SimpleLandlord(simpleLeaselistener);
   try
     {
      persistentState.setProperty("groups", "public, Suran");
      persistentState.setProperty("locators", "jini://localhost, jini://127.0.0.1");

      proxy = new TelephoneProxy(this);

      landLord = new SimpleLandlord(simpleLeaselistener);

      loadMe();

      Entry[] attrSets = (Entry[])persistentState.get("attr");

      init(proxy, attrSets, this, firstLocator);


     }
   catch(Exception e)
     {
      Logger.err.println(e.getMessage());
      e.printStackTrace(Logger.err);
     }

  }

  private void loadMe()
  {
      String filename = System.getProperty("jaugment2.openh323telephone.TelephoneService.persistentStateFilename");
      if(filename==null)
        {
         filename = "TelephoneServiceState.dat";
         System.setProperty("jaugment2.openh323telephone.TelephoneService.persistentStateFilename", filename);
        }

      persistentState.setProperty("persistentStateFilename", filename);


   load();


   if(persistentState.get("attr")==null)
     {
      persistentState.setProperty("exclusiveRessourceName", System.getProperty("jaugment2.openh323telephone.TelephoneService.exclusiveRessourceName").toString());
      Entry[] attrSets = new Entry[]
                    {
                     new net.jini.lookup.entry.Name("Telephone"),
                     new net.jini.lookup.entry.Comment("A telephone using openh323"),
                     new net.jini.lookup.entry.ServiceInfo("TelephoneService", "Marcus Wolschon <Suran@gmx.net>", "Marcus Wolschon <Suran@gmx.net>", "0.0.1", "standart", "00000"),
                     TelephoneARRepresentation.getUIDescriptor(),
                     TelephoneSwingRepresentation.getUIDescriptor(),
//TODO: Text-representartion
                     new ExclusiveRessourceUserEntry( persistentState.getProperty("exclusiveRessourceName"), "hangup", new Class[0], new Object[0] )
                    };
      persistentState.put("attr", attrSets);
     }

   save();

  }



  //------------------------------------

  private boolean inCall = false;
  private Process proc = null;;


  public void hangup()
  {
   if(inCall)
     {
      try
        {
         if(proc != null)
           {
            //hang up/end waiting for calls first
            proc.getOutputStream().write("\nX\n".getBytes());
            proc.destroy();
            proc = null;
            inCall = false;
           }
        }
      catch(Exception e)
        {
        }
     }
  }

  /**
   * Make a call to "to"
   */
  public boolean makeCall(String to)
  {
   try
     {
        // get the esxclusive ressource
      Enumeration enum = exclusiveUsers.elements();
      while(enum.hasMoreElements())
        {
         ExclusiveUsersEntry e = (ExclusiveUsersEntry)enum.nextElement();
         if(debugMessages)Logger.debug.println("DEBUG: [TelephoneService] claiming ressource");
         e.claimRessource();
        }


         //hang up/end waiting for calls first
      if(proc != null)
        {
         proc.getOutputStream().write("\nX\n".getBytes());
         proc.destroy();
         proc = null;
         inCall = false;
        }

      String cmdln = System.getProperty("jaugment2.openh323telephone.TelephoneService.callcmdln");
      if(cmdln == null)
        cmdln = "/home/download/openh323/oh-phone_1.1_linux_x86 --videoreceive x11";
      proc = Runtime.getRuntime().exec(cmdln+" "+to);
      InputStream err = proc.getErrorStream();
      InputStream in = new BufferedInputStream(proc.getInputStream(), 2048);
      OutputStream out = proc.getOutputStream();
      Thread.sleep(1000);
      int aval = 0;//in.available();
//      for(int i=0; i<5; i++)
/*      if(aval==0)
        {
         Thread.sleep(1000);
         aval = in.available();
        };
     byte retval[] = new byte[aval];

     if(in.read(retval) != aval)
       {if(debugMessages)Logger.debug.println("DEBUG: [RememberanceAgentService:getDocument("+docid+")] warning! did not reat specified number of bytes");}
*/
     aval = err.available();
     if(aval>0)
       {
        byte retvalerr[] = new byte[aval];
        err.read(retvalerr);

        Logger.err.println("ERROR: [TelephoneService] Maybe I could not connect "+(new String(retvalerr)));
        out.write("\nX\n".getBytes());
        proc.destroy();
        return(false);
       };
     inCall = true;
     return(true);
    }
  catch(Exception e)
    {
     e.printStackTrace(Logger.err);
    }
   return(false);
  };


  /**
   * Return the directory-services found so far
   */
  public LinkedList getDirectories()
  {
   LinkedList retval = new LinkedList();
   Enumeration enum = directories.elements();
   while(enum.hasMoreElements())
        {
         retval.add(enum.nextElement());
        }
   return(retval);
  }

}
