/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: openh323-telephone
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2000-12-17, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2000-12-17, Marcus Wolschon<Suran@gmx.net>)
  *       - added directories (not used yet)
  *       - gets directories at startup
  *       - registers for and handles events
  *       - uses directories
  * 0.0.3 (2000-12-19, Marcus Wolschon<Suran@gmx.net>)
  *       - proxy is a TelephoneProxyIF
  *       - TelephoneSwingRepresentationListener
  * 0.0.4 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *       - reworked searchDirectories to share code with AR-representation
  *       - localized
  * -------------------------------------------------------------------
  * TODO:
  *       - allow incomming calls
  *       - allow video via ppm-files
  */
package jaugment2.openh323telephone;

  //JDK
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.MarshalledObject;
import java.lang.StringBuffer;
import java.lang.Package;
import java.lang.Double;
import java.io.Serializable;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.lang.IllegalStateException;
import java.util.HashSet;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;
import java.util.ListIterator;
import java.util.LinkedList;
import java.beans.*;
   //Swing
import javax.swing.*;
import javax.swing.JScrollPane;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import java.awt.event.*;
import java.awt.*;
  //JNDI
import javax.naming.NamingEnumeration;
import javax.naming.directory.SearchResult;
import javax.naming.directory.Attributes;
import javax.naming.directory.Attribute;
  //JINI
import net.jini.core.entry.Entry;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
import net.jini.core.event.EventRegistration;
import net.jini.lease.LeaseRenewalManager;
import net.jini.core.lease.Lease;
import net.jini.core.lookup.ServiceItem;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JComponentFactory;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
import net.jini.lookup.ui.attribute.RequiredPackages;
  //project
import jaugment2.UI.swingUI.SwingUIRepresentable;
import jaugment2.helpers.Logger;
import jaugment2.directory.Directory;
import jaugment2.directory.Directory.*;
import jaugment2.openh323telephone.TelephoneService.TelephoneServiceEvent;

/** A telephone using openh323
  */
public class TelephoneSwingRepresentation  extends JPanel implements Entry, SwingUIRepresentable, RemoteEventListener
{
 private static jaugment2.helpers.Translator translate = new jaugment2.helpers.Translator(TelephoneSwingRepresentation.class);
 private static final boolean debugMessages = true;
 protected TelephoneProxyIF proxy;
 private LinkedList directories = new LinkedList();


  JButton jButton1 = new JButton();
  JButton jButton2;
  JComboBox jComboBox_to;
  JButton jButton3;

  /** React to events from our service.
    */
  public void notify(RemoteEvent evt) throws UnknownEventException, RemoteException
  {
   Logger.debug.println("DEBUG[TelephoneSwingRepresentation] notify()");
   if(!(evt instanceof TelephoneServiceEvent))
     throw(new UnknownEventException("expected TelephoneServiceEvent"));

   switch((int)((TelephoneServiceEvent)evt).getID())
    {
     case TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_FOUND:
       {
        directories.add(((TelephoneServiceEvent)evt).obj);
        searchDirectories(true, (DefaultComboBoxModel)jComboBox_to.getModel(), directories);
        break;
       }
     case TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_GONE:
       {
        directories.remove(((TelephoneServiceEvent)evt).obj);
        break;
       }
     default:
        throw(new UnknownEventException("unknown event-ID"));
    }
  }


 /**
  * Get a UIDescriptor as needed by the serviceui-draft-standart
  */
 public static UIDescriptor getUIDescriptor()
  {
   try
     {
      return(new UIDescriptor(MainUI.ROLE,
                              JComponentFactory.TOOLKIT,
                              getServiceUIProperties(),
                              new MarshalledObject(new Factory()))
                             );
     }
   catch(IOException e)
     {
      e.printStackTrace(Logger.err);
     }
   return(null);
  }



  /**
   * Gets a serviceui-draft-standart-compactible attribute-set of this representation
   */
  static Set getServiceUIProperties()
  {
   HashSet set = new HashSet();
   set.add("javax.swing.JComponent");
   set.add("javax.swing.JPanel");
   set.add("java.awt.Component");

   HashSet retval = new HashSet();
   retval.add(new UIFactoryTypes(set));

   HashMap map = new HashMap();
   map.put("javax.swing", Package.getPackage("javax.swing").getSpecificationVersion());
   RequiredPackages req = new RequiredPackages(map);
   retval.add(req);

   return(retval);
  }


  /**
   * A serializable factory for creating this UI
   */
  public static class Factory implements JComponentFactory, Serializable
  {


   /**
    * Returns a <CODE>JComponent</CODE>.
    */
   public JComponent getJComponent(Object roleObject)
   {
    ServiceItem item = (ServiceItem)roleObject;

    if(debugMessages)Logger.debug.println("DEBUG: [TelephoneServiceSwingRepresentation.factory] getJComponent called");
    try
      {
       TelephoneSwingRepresentation rep = new TelephoneSwingRepresentation( ((TelephoneProxyIF)item.service) );
       return(rep.getPanel());
      }
    catch(RemoteException ex)
       {return(null);}

   }
  }


  public JPanel getPanel() throws RemoteException
  {
   return(this);
  }



  private LeaseRenewalManager lrm = new LeaseRenewalManager();

  /**
   * Constructor
   */
 public TelephoneSwingRepresentation(TelephoneProxyIF proxy)
 {
  try
    {
     if(debugMessages)Logger.debug.println("DEBUG: [TelephoneSwingRepresentation] TelephoneSwingRepresentation(...)");

     this.proxy = proxy;
     Lease l = proxy.addListener(new TelephoneSwingRepresentationListener(this), null, TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_FOUND | TelephoneServiceEvent.EVID_DIRECTORY_SERVICE_GONE);
     if(l != null)
       {
        lrm.renewFor(l, Lease.FOREVER, null);
       };

     this.directories = proxy.getDirectories();
     searchDirectories(true, (DefaultComboBoxModel)jComboBox_to.getModel(), directories);

     jbInit();
    }
  catch(Exception e)
    {
     Logger.err.println("Exception in TelephoneSwingRepresentation::TelephoneSwingRepresentation() has the Message: "+e.getMessage());
     e.printStackTrace(Logger.err);
    }
 }


  /** Create UI.
    */
  private void jbInit() throws Exception
  {
    jButton2 = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    jComboBox_to = (JComboBox) Beans.instantiate(getClass().getClassLoader(), JComboBox.class.getName());
    jButton3 = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    jButton1.setText(translate.getString("call"));
    jButton1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButton1_actionPerformed(e);
      }
    });
    jButton2.setText(translate.getString("hangup"));
    jButton2.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton2_actionPerformed(e);
      }
    });
    jButton3.setText(translate.getString("search"));
    jButton3.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton3_actionPerformed(e);
      }
    });
    jComboBox_to.setEditable(true);
    this.add(jButton1, null);
    this.add(jComboBox_to, null);
    this.add(jButton3, null);
    this.add(jButton2, null);
  }

  void jButton1_actionPerformed(ActionEvent evt)
  {
   try
     {
      Object to = jComboBox_to.getSelectedItem();
      proxy.makeCall(to.toString());
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }

  void jButton2_actionPerformed(ActionEvent evt)
  {
   try
     {
      proxy.hangup();
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }


  void jButton3_actionPerformed(ActionEvent e)
  {
   searchDirectories(false, (DefaultComboBoxModel)jComboBox_to.getModel(), directories);
  }
  //----------------------------------------------------------




 /**
  * helper-class
  */
 public static class TelAddr
 {
  private String to;
  public String getTo()
  {
   return(to);
  }

  public TelAddr(String to, String displayname)
  {
   this.to = to;
   this.displayname = displayname;
  }

  private String displayname;
  public String toString()
  {
   return((displayname==null?to:displayname+" <"+to+">") );
  }

  public boolean equals(Object o)
  {
   if(o == null)
     return(false);

   if(o instanceof TelAddr)
     {
      return(
              ((TelAddr)o).toString().equals(toString())
            );
     }
   else
     {
      return(o.toString().equals(to));
     }
  }

 };


  protected static void searchDirectories(boolean local, javax.swing.DefaultComboBoxModel model, LinkedList directories)
  {
  if(debugMessages)Logger.debug.println("DEBUG: [TelephoneSwingRepresentation] searchDirectories() has got "+directories.size()+" directories to search");
   for(int d=0; d<directories.size(); d++)
     {
      try
        {
         Directory dir = (Directory)directories.get(d);
         SearchResult enum[];
         if(local)
           {
            enum = dir.simpleSearch("(&(telephone=*))", Directory.LEVEL_LOCAL);
           }
         else
           {
            enum = dir.simpleSearch("(&(cn="+model.getSelectedItem().toString()+"*)(telephone=*))", Directory.LEVEL_ALL);
           }
         if(debugMessages)Logger.debug.println("DEBUG: [TelephoneSwingRepresentation] got "+enum.length+" results");
          for(int i=0; i<enum.length; i++)
            {
             SearchResult res = enum[i];
             try
               {
                Attribute cn   = res.getAttributes().get("cn");
                Attribute tel = res.getAttributes().get("telephone");
                if(cn != null)
                if(tel != null)
                  {
                   if(debugMessages)Logger.debug.println("DEBUG: [TelephoneSwingRepresentation] 1 result passed");
                        // get the telephone-number
                   TelAddr addr = new TelAddr(tel.get(0).toString(), cn.get(0).toString());
                 boolean isin = false;
                 for(int j=0; j<model.getSize(); j++)
                    {
                     if(model.getElementAt(j) != null)
                     if(model.getElementAt(j) instanceof TelAddr)
                     if(((TelAddr)model.getElementAt(j)).equals(addr))
                       {
                        isin = true;
                        break;
                       }
                    }

                    // if we do not yet have this person
                 if(!isin)
                   {
                    model.addElement(addr);
                   }

                 }
              }
             catch(Exception ex)
              {
               Logger.err.println("ERROR: [TelephoneSwingRepresentation] Exception thrown while investigating the resulta of a directory-search");
               ex.printStackTrace(Logger.err);
              }
            }//return-values
        }
      catch(Exception ex)
        {
         Logger.err.println("ERROR: [TelephoneSwingRepresentation] Exception thrown while earching the directories");
         ex.printStackTrace(Logger.err);
        }
     }
  }




}
